/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

(function(){
dojo.provide("com.ibm.rdm.web.rrcx.artifact.ElementCommentCache");

dojo.require("com.ibm.rdm.web.commenting.EventTopics");
dojo.require("com.ibm.rdm.web.notifications.EventTopics");
dojo.require("com.ibm.rdm.web.notifications.ChangeNotifier");
dojo.require("com.ibm.rdm.web.commenting.CommentingClient");
dojo.require("com.ibm.rdm.web.commenting.Comment");

dojo.declare("com.ibm.rdm.web.rrcx.artifact.ElementCommentCache", null, {

	_artifact: null,
	_elementComments: [],
	_listener: null,

    // Constructor
    constructor: function(artifact) {
		this._artifact = artifact;
		var deferred = com.ibm.rdm.web.commenting.CommentingClient.getComments(this._artifact.getResourceURI(), null, false);
		deferred.addBoth(this, function(feed) {
			this._elementComments = [];
			if (feed && feed.doc) {
				var entries = feed.doc.getElementsByTagName("entry");
				for (var i = 0; i < entries.length; i++) {
					var comment = new com.ibm.rdm.web.commenting.Comment();
					comment.parseFeedEntry(entries[i], com.ibm.rdm.web.FRONTING_SERVICE_COMMENTS);
					if (!comment.anyErrors() && comment.locationId != null) {
						this._elementComments.push(comment);
					}
				}
			}
		});
		this._listener = com.ibm.rdm.web.notifications.ChangeNotifier.addListener({
			eventTopic: com.ibm.rdm.web.commenting.EventTopics.COMMENT_SAVED_TOPIC,
			callbackFunction: this._updateCache,
			callbackScope: this
		});
    },
	
	_updateCache: function(params) {
		if (params.notifierParams.eventTopic == com.ibm.rdm.web.commenting.EventTopics.COMMENT_SAVED_TOPIC) {
			if (params.notifierParams.comment && params.notifierParams.comment.locationId != null && params.notifierParams.comment.resourceUri == this._artifact.getResourceURI()) {
				this._elementComments.push(params.notifierParams.comment);
				com.ibm.rdm.web.notifications.ChangeNotifier.notifyListeners({
					eventTopic: com.ibm.rdm.web.notifications.EventTopics.ELEMENT_CACHE_UPDATED,
					resourceUriScope: this._artifact.getResourceURI(),
					type: "comment"
				});
			}
		}
	},
	
	destroy: function() {
		com.ibm.rdm.web.notifications.ChangeNotifier.removeListener(this._listener);
	},
	
	getElementComments: function(locationId) {
		var elementComments = [];
		for (var i = 0; i < this._elementComments.length; i++) {
			var comment = this._elementComments[i];
			if (comment.locationId.lastIndexOf(locationId) == comment.locationId.length - locationId.length) {
				elementComments.push(comment);
			}
		}
		return elementComments;
	}
	
});
}());