/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

(function(){
dojo.provide("com.ibm.rdm.web.rrcx.artifact.ElementLinkCache");

dojo.require("com.ibm.rdm.web.notifications.EventTopics");
dojo.require("com.ibm.rdm.web.notifications.ChangeNotifier");
dojo.require("com.ibm.rdm.web.server.client.utils.LinkUtils");

dojo.declare("com.ibm.rdm.web.rrcx.artifact.ElementLinkCache", null, {

	_artifact: null,
	_elementLinks: [],
	_listener: null,

    // Constructor
    constructor: function(artifact) {
		this._artifact = artifact;
		var params = {
			sourceAndTargetURI: this._artifact.getResourceURI()
		};
		var deferred = com.ibm.rdm.web.server.client.utils.LinkUtils.getLinksForResource(params);
		deferred.addBoth(this, function(links) {
			this._elementLinks = [];
			if (links != null) {
				for (var i = 0; i < links.length; i++) {
					var link = links[i];
					if (link.isElementLink()) {
						this._elementLinks.push(link);
					}
				}
			}
		});
		this._listener = com.ibm.rdm.web.notifications.ChangeNotifier.addListener({
			eventTopics: [com.ibm.rdm.web.notifications.EventTopics.LINK_CREATED, com.ibm.rdm.web.notifications.EventTopics.LINK_DELETED],
			callbackFunction: this._updateCache,
			callbackScope: this
		});
    },
	
	_updateCache: function(params) {
		if (params.notifierParams.eventTopic == com.ibm.rdm.web.notifications.EventTopics.LINK_CREATED) {
			if (params.notifierParams.links && params.notifierParams.links.length > 0) {
				for (var i = 0; i < params.notifierParams.links.length; i++) {
					var link = params.notifierParams.links[i];
					if (link.isElementLink()) {
						var allowAdd = true;
						for (var x = 0; x < this._elementLinks.length; x++) {
							var elementLink = this._elementLinks[x];
							if (elementLink.getResourceURI() == link.getResourceURI()) {
								allowAdd = false;
								break;
							}
						}
						if (allowAdd) {
							this._elementLinks.push(link);
							com.ibm.rdm.web.notifications.ChangeNotifier.notifyListeners({
								eventTopic: com.ibm.rdm.web.notifications.EventTopics.ELEMENT_CACHE_UPDATED,
								resourceUriScope: this._artifact.getResourceURI(),
								type: "link"
							});
						}
					}
				}
			}
		} else if (params.notifierParams.eventTopic == com.ibm.rdm.web.notifications.EventTopics.LINK_DELETED) {
			if (params.notifierParams.resource) {
				var linkToRemove = null;
				for (var x = 0; x < this._elementLinks.length; x++) {
					var elementLink = this._elementLinks[x];
					if (elementLink.getResourceURI() == params.notifierParams.resource) {
						linkToRemove = x;
						break;
					}
				}
				if (linkToRemove != null) {
					this._elementLinks.splice(linkToRemove, 1);
					com.ibm.rdm.web.notifications.ChangeNotifier.notifyListeners({
						eventTopic: com.ibm.rdm.web.notifications.EventTopics.ELEMENT_CACHE_UPDATED,
						resourceUriScope: this._artifact.getResourceURI(),
						type: "link"					
					});
				}
			}
		}
	},
	
	destroy: function() {
		com.ibm.rdm.web.notifications.ChangeNotifier.removeListener(this._listener);
	},
	
	getElementLinks: function(locationId) {
		var elementLinks = [];
		for (var i = 0; i < this._elementLinks.length; i++) {
			var link = this._elementLinks[i];
			if (link.getSubjectLocationId() == locationId || link.getObjectLocationId() == locationId) {
				elementLinks.push(link);
			}
		}
		return elementLinks;
	}
	
});
}());