/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

(function(){
dojo.provide("com.ibm.rdm.web.rrcx.artifact.RRCXArtifactHandler"); 

dojo.require("com.ibm.rdm.web.rrcx.common.PluginEditorHandle"); 
dojo.require("com.ibm.rdm.web.rrcx.artifact.RRCxArtifact");
dojo.require("com.ibm.rdm.web.ext.ArtifactHandler"); 
dojo.require("com.ibm.rdm.web.model.resources.ResourceManager");
dojo.require("com.ibm.rdm.web.model.types.RRMTypes");
dojo.require("com.ibm.rdm.web.viewer.viewerutils");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.server.client.utils.ResourceUtil");
dojo.require("com.ibm.rdm.web.notifications.ChangeNotifier");
dojo.require("com.ibm.rdm.web.viewer.EmbeddedDiagramViewer");
dojo.require("jazz.ui.HoverPopup");
dojo.require("com.ibm.rdm.web.ext.ElementRichHover");
dojo.require("com.ibm.rdm.web.links.Link");
dojo.require("com.ibm.rdm.web.common.MessageHandler");

dojo.requireLocalization("com.ibm.rdm.web.rrcx.artifact", "RRCXArtifactHandler");

var EmbeddedDiagramViewer = com.ibm.rdm.web.viewer.EmbeddedDiagramViewer;
var viewerutils = com.ibm.rdm.web.viewer.viewerutils;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var ChangeNotifier = com.ibm.rdm.web.notifications.ChangeNotifier;
/**
   This is the object that the extension providers have to implement
 */
dojo.declare("com.ibm.rdm.web.rrcx.artifact.RRCXArtifactHandler", com.ibm.rdm.web.ext.ArtifactHandler, {

	_elementLinkCache: null,
	_elementCommentCache: null,
	_elementRichHovers: [],
	_converterDeferred: null,
	_messages: dojo.i18n.getLocalization("com.ibm.rdm.web.rrcx.artifact", "RRCXArtifactHandler"),

    // Constructor
    constructor: function() {
		//console.debug("Concrete class constructor called");
    },
    
    // Public methods

    // Retrieves the artifact type
    create :  function(projectURI, folderURI, objectTypeURI, title, description, openResourceInEditMode, options, resourceUriScope) {

   		var deferred = new dojo.Deferred();
   		var artifactFormat = com.ibm.rdm.web.model.types.RRMTypes.getArtifactFormats(projectURI)[this._entry.id];
		var resourceObject  = this.instantiateResource(
			{
				entityType: artifactFormat,
				projectURI: projectURI,
				options: options,
				objectTypeURI: objectTypeURI
			}
		);
		
		var callbackHandler = com.ibm.rdm.web.server.client.utils.ResourceUtil;
		
		resourceObject.setProjectURI(projectURI);
		resourceObject.setTitle(title);
		resourceObject.setDescription(description);

		this._initializeResource(resourceObject, options);
		
		var addTypeDeferred = resourceObject.addType(objectTypeURI);
		addTypeDeferred.addCallback(this,function(typeResponse){
		var createArtifactDeferred = resourceObject.save({refreshAfterSave: true, softRefresh: false});
			createArtifactDeferred.addCallback(this, function(createResponse){
			//move resource to folder
			var moveToFolderDeferred = callbackHandler._performMoveOperation(projectURI, resourceObject.resourceURI, folderURI, false);
			moveToFolderDeferred.addBoth(this, function(moveResponse){
				deferred.callback(resourceObject);
				callbackHandler.publishResourceCreatedEvent(openResourceInEditMode == "undefined" ? true : openResourceInEditMode, resourceObject.resourceURI, projectURI, resourceObject, resourceUriScope);
			});
		});
			createArtifactDeferred.addErrback(this,function(createErrback){
				deferred.callback(createErrback);
				return createErrback;
			});
			return typeResponse;
		});
		
		addTypeDeferred.addErrback(this,function(errback){
			deferred.callback(errback);
			return errback;
		});
		return deferred;
    
    },
    
    _initializeResource: function(resourceObject){
    		
    },
    
   getEmbeddedResourceViewer : function(params)
    {
    	var embeddedResourceViewer = new EmbeddedDiagramViewer(
    			params.node, 
    			params.response, 
    			params.embeddedLevel, 
    			params.options,
    			params.projectURI,
    			params.reviewParser, 
    			params.projectSnapshotURI, 
    			params.resourceURI);
        return embeddedResourceViewer;
    },
    
    instantiateResource : function(params)
    {
    		var resourceObject  = new com.ibm.rdm.web.rrcx.artifact.RRCxArtifact(params);
    		return resourceObject;
    },
	
    // Retrieves the artifact type
    load : function(resourceViewerWidget, webPageHandle)
    {	
    	if(!this.pluginEditorHandle) {
	    	this.resourceViewerWidget = resourceViewerWidget;
	    	this.contentType = resourceViewerWidget.resourceViewer._contentType;
	    	this.revision = resourceViewerWidget.resourceViewer.revision;
	    	this.pluginEditorHandle = new com.ibm.rdm.web.rrcx.common.PluginEditorHandle(webPageHandle, this);
	    	this.pluginEditorHandle.setResourceViewerWidget(resourceViewerWidget);
    	}
    },
    
    viewResource : function(resourceViewerWidget, webPageHandle) 
    {
    	if(!this.pluginEditorHandle)
    		this.load(resourceViewerWidget, webPageHandle);
		
		this.resourceViewerWidget._contentPane.domNode.style.paddingTop = "0px";
		this.resourceViewerWidget._contentPane.domNode.style.paddingRight = "0px";
		this.resourceViewerWidget._contentPane.domNode.style.paddingBottom = "0px";
		this.resourceViewerWidget._contentPane.domNode.style.paddingLeft = "0px";
			
		if (!this.resourceViewerWidget.resourceViewer.editMode) {
			this._removeHovers();
			this.viewRRCXResource(resourceViewerWidget, webPageHandle);

		} else {
			this.resourceViewerWidget.setDocumentHasLoaded(true);
			this.resourceViewerWidget.setDimensions();
		}
		
        return this.pluginEditorHandle;
    	
    },
	
	viewRRCXResource: function(resourceViewerWidget, webPageHandle) {
    	this._converterDeferred = null;
		var onLoaded = dojo.hitch(this, function(resourceObj) {
			var content = "&nbsp;";
			var statusCode = resourceObj.ioArgs ? resourceObj.ioArgs.xhr.status : 
					resourceObj.dojoType === "cancel" ? 999 : 111;
			switch (statusCode) {
				case 200:	// Success
				case 304:	// Cached
					content = viewerutils.transformImageGenerationDocument(resourceObj.ioArgs.xhr.responseText);

					this.resourceViewerWidget._contentPane.domNode.style.position = "absolute";
					this.resourceViewerWidget._contentPane.attr('content', content);
				
					this.doResourceSpecificLoad(resourceObj);

					var idnodes = dojo.query("[id]", this.resourceViewerWidget._contentPane.domNode);
					this.resourceViewerWidget.connectIdNodeOnClick(idnodes);

					this.resourceViewerWidget.setDocumentHasLoaded(true);
					this.resourceViewerWidget.setDimensions();
					this._addRichHovers();
					break;
				case 404:	// Converter service is not available
				case 504:
					this.resourceViewerWidget._contentPane.attr('content', content);
					this.resourceViewerWidget.setDocumentHasLoaded(true);
					this.resourceViewerWidget.setDimensions();				
					com.ibm.rdm.web.common.MessageHandler.showError("8011");
					break;
				case 505:	// Failed Dependency (slightly misused here to indicate that the converter is not on a supported platform)
					this.resourceViewerWidget._contentPane.attr('content', content);
					this.resourceViewerWidget.setDocumentHasLoaded(true);
					this.resourceViewerWidget.setDimensions();				
					com.ibm.rdm.web.common.MessageHandler.showError("8012");
					break;
				case 999:	// Deferred has been cancelled due to entering edit mode
					this.resourceViewerWidget._contentPane.attr('content', content);
					this.resourceViewerWidget.setDocumentHasLoaded(true);
					this.resourceViewerWidget.setDimensions();
					break;
				default:	// Any other status is an error
					this.resourceViewerWidget._contentPane.attr('content', content);
					this.resourceViewerWidget.setDocumentHasLoaded(true);
					this.resourceViewerWidget.setDimensions();				
					com.ibm.rdm.web.common.MessageHandler.showError("8010");
			} 
		});

		this.revision = resourceViewerWidget.resourceViewer.revision;
		var urlToUse = resourceViewerWidget._getConversionURL(
				this.resourceViewerWidget.resourceURI, this.contentType, this.revision);
		this._converterDeferred = RDMClient.doGet({
			uri: urlToUse,
			skipErrors: true
		});
		
		this._converterDeferred.addBoth(this, onLoaded);
	},
	
	doResourceSpecificLoad: function(resourceObj) {
		// summary:
		//    A function for resource handlers to override to allow
		//	  for resource-specific actions to take place during the 
		//	  loading of the resource.
		
	},
	
	getConverterDeferred: function() {
		return this._converterDeferred;
	},
	
	_addRichHovers: function() {
		var diagramElements = dojo.query(".diagramElement");
		for (var i = 0; i < diagramElements.length; i++) {
			var diagramElement = diagramElements[i];
			this._createHover(diagramElement, this.resourceViewerWidget.getResourceObject());
		}
	},
	
	_createHover: function(diagramElement, resource) {
		var type = "";
		var label = "";
		var description = "";
		if (dojo.attr(diagramElement, "controltype") != null) {
			//TODO: Remove the second replace which removes the whitespace in the type
			// once we can put in a proper fix for this post GA
			type = dojo.attr(diagramElement, "controltype").replace(/\+/g, " ").replace(/\ /g, "");
			//TODO: WI43336
			if (type == "Message") {
				type = "MessageFlow";
			} else if (type == "Drop-DownList") {
				type = "DropDownList";
			}
		}
		if (dojo.attr(diagramElement, "controllabel") != null) {
			label = dojo.attr(diagramElement, "controllabel").replace(/\+/g, " ");
		}
		if (dojo.attr(diagramElement, "controlDesc") != null) {
			description = dojo.attr(diagramElement, "controlDesc").replace(/\+/g, " ");
		}
		var name = this._messages[type];
		if (label != "") {
			name += ': ' + com.ibm.rdm.web.util.queryutils.decode(label);
		}
		var elementLinks = dojo.query("#" + diagramElement.id + " > ul.elementLinks > li > a");
		var genericLinks = [];
		for (var i = 0; i < elementLinks.length; i++) {
			var elementLink = elementLinks[i];
			var link = new com.ibm.rdm.web.links.Link({
				subject: this.resourceViewerWidget.resourceURI,
				object: elementLink.href,
				subjectLocationId: diagramElement.id
			});
			genericLinks.push(link);
		}
		var elementRichHover = new jazz.ui.HoverPopup({
			around: diagramElement,
			createContents: dojo.hitch(this, function(pop) {
				var elementHover = new com.ibm.rdm.web.ext.ElementRichHover({
					resourceUri: resource.getResourceURI(),
					elementDescription: description,
					elementName: name,
					artifact: resource,
					locationId: diagramElement.id,
					genericLinks: genericLinks
				});
				this._elementRichHovers.push(elementHover);
				return elementHover.domNode;
			})
		});
		this._elementRichHovers.push(elementRichHover);
	},
	
	_removeHovers: function() {
		dojo.forEach(this._elementRichHovers, function(richHover){
			if (richHover.destroyRecursive && !richHover._beingDestroyed){
				richHover.destroyRecursive();
			}
			else if (richHover.destroy && !richHover._beingDestroyed){
				richHover.destroy();
			}
			delete (richHover);
		});
	},
	
	_generateUuids: function(numIds) {
		var uuids = [];
		for (var i=0; i < numIds; i++) {
			uuids.push(dojox.uuid.generateRandomUuid());
		}
		return uuids;
	}
});
}());