/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.rrcx.artifact.RRCxArtifact");
dojo.require("com.ibm.rdm.web.ext.Artifact");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.model.resources.requirements.ResourceParser");
dojo.require("com.ibm.rdm.web.rrcx.artifact.ElementLinkCache");
dojo.require("com.ibm.rdm.web.rrcx.artifact.ElementCommentCache");
	
var queryutils = com.ibm.rdm.web.util.queryutils;
var domutils = com.ibm.rdm.web.util.domutils;

var ResourceParser = com.ibm.rdm.web.model.resources.requirements.ResourceParser;

dojo.declare("com.ibm.rdm.web.rrcx.artifact.RRCxArtifact", com.ibm.rdm.web.ext.Artifact,
    {
    	_content: null,
		_elementLinkCache: null,
		_elementCommentCache: null,
		_idAttribute: ["id"],
    	_createXMLBody: function(){

    		var parser = {
	
	    		_parseObjectSpecificContent: function(artifact) {
	    			var objectXML = '\n<rm:rrcContent>\n';
					var text = artifact.getContent();
					objectXML += com.ibm.rdm.web.util.domutils.getXMLEncodedText(text);
				
					objectXML += '\n</rm:rrcContent>\n';
	    	
	    			return objectXML;
	   			 } , 
	     
	    		_parseObjectSpecificContentOutsideArtifact: function(artifact) {   	
	    			return "";
				}  
			};
			
    		return ResourceParser.parseResourceToXML(this, parser);
    	},
    	
    	getContent: function() {
    		return this._content;
    		
    	},  	
    	
    	setContent: function(content) {
    		this._content = content;
    	},
    	
    	load: function() {
    		var deferred = new dojo.Deferred();
    		var loadDeferred = this.inherited("load", arguments);
    		loadDeferred.addCallback(this, function(loadResponse){
    			this.setContent(this._parseRRCContent(this.resourceXmlContent));
				this._clearCaches();
    			deferred.callback(this);
    		});
    		loadDeferred.addErrback(this, function(loadResponse){
				deferred.errback(loadResponse);
			});
			return deferred;
    	},
		
		loadFromXml: function(xmlDom, resourceProperties) {
			this.inherited(arguments);
			this.setContent(this._parseRRCContent(this.resourceXmlContent));
		},
		
    	trim: function(stringToTrim) {
    		return stringToTrim.replace(/^\s+|\s+$/g,"");
    	},
    	
    	clone: function(copyStub){
    		this.inherited(arguments);
    		copyStub.setContent(this._content);
		},		
    	

	    _parseRRCContent: function(xmlDoc) {
			var contentNodes = domutils.getElementsByTagNameNS( 
					xmlDoc,
					"rrcContent",
					queryutils.RM_NAMESPACE,
					queryutils.RM_QUALIFIER
			);
			
			var contentNode = contentNodes[0];
			var contentXml = "";
			for (i=0; i < contentNode.childNodes.length; i=i+1) {
				contentXml = contentXml + dojox.xml.parser.innerXML(contentNode.childNodes[i]);
			}
			contentXml = this.trim(domutils.getXMLDecodedText(contentXml));
			return contentXml;
		},
		
		_clearCaches: function() {
			if (this._elementLinkCache) {
				this._elementLinkCache.destroy();
				this._elementLinkCache = null;
			}
			if (this._elementCommentCache) {
				this._elementCommentCache.destroy();
				this._elementCommentCache = null;
			}			
		},
		
		_reset: function() {
			this.inherited(arguments);
			this._clearCaches();
		},
		
		getElementLinks: function(locationId) {
			if (!this._elementLinkCache) {
				this._elementLinkCache = new com.ibm.rdm.web.rrcx.artifact.ElementLinkCache(this);
			}
			return this._elementLinkCache.getElementLinks(locationId);
		},
	
		getElementComments: function(locationId) {
			if (!this._elementCommentCache) {
				this._elementCommentCache = new com.ibm.rdm.web.rrcx.artifact.ElementCommentCache(this);
			}
			return this._elementCommentCache.getElementComments(locationId);
		},
		
		_getValidElementIds: function(){
			
			var rrcContentDOM =dojox.xml.parser.parse(this._content);		
			var validElementIds = [""];
			
			dojo.forEach(this._idAttribute, function(attribute){
				var nodeList = dojo.query("[" + attribute + "]", rrcContentDOM);				
				dojo.forEach(nodeList, function(node){
					var id = dojo.attr(node, attribute);
					if(node.nodeName == 'sketch:Diagram' || node.nodeName == 'story:frame'){//sketching diagram specific
						if(node.getAttribute('master') != null)// it has embedded sketches, so the diagram layer itself can be linked
							validElementIds.push(id);
					}
					else if(node.nodeName != 'delete')
						validElementIds.push(id);
				});
			});


			return validElementIds;
		},
		
		_getInvalidLinks: function (/*Array*/links) {
			var invalid = [];
			
			var validElementIds = this._getValidElementIds();		
			var linksForElement = new dojox.collections.Dictionary();
			var self = this;
			//sort out links by element id
			dojo.forEach(links, function(link){
				var elementID = link._subject == self.resourceURI ? link._subjectLocationId : link._objectLocationId;
				
				if(elementID != null){
					var index = elementID.lastIndexOf('/') + 1;
					elementID = elementID.substring(index);
					
					var elementLinks = linksForElement.item(elementID);
					if(elementLinks == null){
						elementLinks = [];
						linksForElement.add(elementID, elementLinks);
					}
					
					elementLinks.push(link);
				}
					
			});
			
			//discard all valid links
			dojo.forEach(validElementIds, function(id){
				linksForElement.remove(id);
			});
			
			//concatinate all invalid links
			dojo.forEach(linksForElement.getValueList() , function(deletedElementLinks){
				invalid = invalid.concat(deletedElementLinks);
			});
			return invalid;
		}
    }
);

})();
