/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
// NLS_CHARSET=UTF-8

(function(){
dojo.provide("com.ibm.rdm.web.rrcx.artifact.sketch.SketchEditor");
dojo.require("dijit._Templated");
dojo.require("com.ibm.rdm.web.rrcx.common.PluginEditor");

dojo.require("jazz.ui.Toolbar"); 
dojo.require("jazz.ui.MegaMenu"); 
dojo.require("jazz.ui.toolbar.Button");
dojo.require("jazz.ui.toolbar.Separator"); 
dojo.require("jazz.ui.toolbar.MenuButton"); 
dojo.require("com.ibm.rdm.web.rrcx.common.widgets.ImgMenuItem");
dojo.require("com.ibm.rdm.web.rrcx.common.widgets.MenuItemTitle");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.web.rrcx.artifact.sketch", "SketchEditor");

/**
   This is the object that the extension providers have to implement
 */
dojo.declare("com.ibm.rdm.web.rrcx.artifact.sketch.SketchEditor", 
	[com.ibm.rdm.web.rrcx.common.PluginEditor, dijit._Templated], 
{ 
	widgetsInTemplate:true,
	templatePath : dojo.moduleUrl("com.ibm.rdm.web.rrcx", "artifact/sketch/templates/SketchEditor.html"),
	_messages: dojo.i18n.getLocalization("com.ibm.rdm.web.rrcx.artifact.sketch", "SketchEditor"),
	_imagePath: "com.ibm.rdm.web.rrcx/artifact/sketch/icons/editor/",
	
	_removeMaster: null,
	
	postCreate : function() {
		this.inherited(arguments);
		
		if (this._paletteToolbar) {
			
			var self = this;
			
			this._removeMaster = new jazz.ui.toolbar.Button({
				title: this._messages.Remove_Master_Tooltip,
				label: this._messages.Remove_Master_Label,
				enabled: true,
				onClick: dojo.hitch(this, function(event) {
					dojo.stopEvent(event);
					self.pluginHandle.invokeEditorAction("sid.master.remove");
				})
			});
			this._firstRowToolbar.addChild(this._removeMaster);
			this.hideRemoveMaster();
			this._toDestroy.push(this._removeMaster);
		}
	},
	
	showRemoveMaster : function() {
		if (this._removeMaster)
			this._removeMaster._setVisibleAttr(true);
	},
	
	hideRemoveMaster : function() {
		if (this._removeMaster)
			this._removeMaster._setVisibleAttr(false);
	},
	
	/**
	 * get a list of the items.
	 * this is un-ordered!
	 */	
	getPluginItems : function (){
		return [
		{	id: "com.ibm.sid.ui.sketch.actions.AddSelectionToToolBar",
			factory: "",
			tool: "com.ibm.sid.ui.tools.ExSelectionTool"
		},
		{	id: "com.ibm.sid.ui.sketch.actions.AddMarqueeToToolBar",
			factory: "",
			tool: "com.ibm.sid.ui.sketch.tools.CustomMarqueeTracker"
	  	},
		{	id: "com.ibm.sid.ui.sketch.actions.AddNoteToToolBar",
			factory: "com.ibm.rdm.ui.gef.palette.EMFCreationFactory",
			tool: "com.ibm.sid.ui.tools.SketchingCreationTool"
		},
		{	id: "com.ibm.sid.ui.sketch.actions.AddCursorToToolBar",
			factory: "com.ibm.rdm.ui.gef.palette.EMFCreationFactory",
			tool: "com.ibm.sid.ui.tools.SketchingCreationTool"
		},
	  	{	id: "com.ibm.sid.ui.sketch.actions.AddLabelToToolBar",
	        factory: "com.ibm.rdm.ui.gef.palette.EMFCreationFactory",
	        tool: "com.ibm.sid.ui.tools.SketchingCreationTool"
   	    },
   	    {	id: "com.ibm.sid.ui.sketch.actions.AddParagraphToToolBar",
	        factory: "com.ibm.rdm.ui.gef.palette.EMFCreationFactory",
	        tool: "com.ibm.sid.ui.tools.SketchingCreationTool"
   	    },
	    {	id: "com.ibm.sid.ui.sketch.actions.AddImagePlaceholderToToolBar",
	        factory: "com.ibm.rdm.ui.gef.palette.EMFCreationFactory",
	        tool: "com.ibm.sid.ui.tools.SketchingCreationTool"
	  	},
	    {	id: "com.ibm.sid.ui.sketch.actions.AddTextFieldToToolBar",
	        factory: "com.ibm.rdm.ui.gef.palette.EMFCreationFactory",
	        tool: "com.ibm.sid.ui.tools.SketchingCreationTool"
		},
		{	id: "com.ibm.sid.ui.sketch.actions.AddTextAreaToToolBar",
	        factory: "com.ibm.rdm.ui.gef.palette.EMFCreationFactory",
	        tool: "com.ibm.sid.ui.tools.SketchingCreationTool"
   	    },
		{	id: "com.ibm.sid.ui.sketch.actions.AddButtonToToolBar",
	        factory: "com.ibm.rdm.ui.gef.palette.EMFCreationFactory",
	        tool: "com.ibm.sid.ui.tools.SketchingCreationTool"
   	    },
  	    {	id: "com.ibm.sid.ui.sketch.actions.AddHyperlinkToToolBar",
	        factory: "com.ibm.rdm.ui.gef.palette.EMFCreationFactory",
	        tool: "com.ibm.sid.ui.tools.SketchingCreationTool"
   	    },
  	    {	id: "com.ibm.sid.ui.sketch.actions.AddCheckBoxToToolBar",
	        factory: "com.ibm.rdm.ui.gef.palette.EMFCreationFactory",
	        tool: "com.ibm.sid.ui.tools.SketchingCreationTool"
  	    },
	    {	id: "com.ibm.sid.ui.sketch.actions.AddRadioButtonToToolBar",
	        factory: "com.ibm.rdm.ui.gef.palette.EMFCreationFactory",
	        tool: "com.ibm.sid.ui.tools.SketchingCreationTool"
		},
	    {	id: "com.ibm.sid.ui.sketch.actions.AddItemToToolBar",
	        factory: "com.ibm.rdm.ui.gef.palette.EMFCreationFactory",
	        tool: "com.ibm.sid.ui.tools.SketchingCreationTool"
  	    },
		{	id: "com.ibm.sid.ui.sketch.actions.AddListBoxToToolBar",
	        factory: "com.ibm.rdm.ui.gef.palette.EMFCreationFactory",
	        tool: "com.ibm.sid.ui.tools.SketchingCreationTool"
   	    },
	    {	id: "com.ibm.sid.ui.sketch.actions.AddDropDownListToToolBar",
	        factory: "com.ibm.rdm.ui.gef.palette.EMFCreationFactory",
	        tool: "com.ibm.sid.ui.tools.SketchingCreationTool"
   	    },
	    {	id: "com.ibm.sid.ui.sketch.actions.AddMenuBarToToolBar",
	        factory: "com.ibm.rdm.ui.gef.palette.EMFCreationFactory",
	        tool: "com.ibm.sid.ui.tools.SketchingCreationTool"
  	    },
	    {	id: "com.ibm.sid.ui.sketch.actions.AddMenuToToolBar",
	        factory: "com.ibm.rdm.ui.gef.palette.EMFCreationFactory",
	        tool: "com.ibm.sid.ui.tools.SketchingCreationTool"
   	    },
		{	id: "com.ibm.sid.ui.sketch.actions.AddContextMenuToToolBar",
	        factory: "com.ibm.rdm.ui.gef.palette.EMFCreationFactory",
	        tool: "com.ibm.sid.ui.tools.SketchingCreationTool"
   	    },
	    {	id: "com.ibm.sid.ui.sketch.actions.AddToolBarToToolBar",
	        factory: "com.ibm.rdm.ui.gef.palette.EMFCreationFactory",
	        tool: "com.ibm.sid.ui.tools.SketchingCreationTool"
		},
	 	{	id: "com.ibm.sid.ui.sketch.actions.AddTreeToToolBar",
	        factory: "com.ibm.sid.ui.sketch.PaletteBuilder.TreeCreationFactory",
	        tool: "com.ibm.sid.ui.tools.SketchingCreationTool"
		},
	    {	id: "com.ibm.sid.ui.sketch.actions.AddTableToToolBar",
	        factory: "com.ibm.sid.ui.sketch.PaletteBuilder.TreeCreationFactory",
	        tool: "com.ibm.sid.ui.tools.SketchingCreationTool"
		},
	    {	id: "com.ibm.sid.ui.sketch.actions.AddTreeTableToToolBar",
	        factory: "com.ibm.sid.ui.sketch.PaletteBuilder.TreeCreationFactory",
	        tool: "com.ibm.sid.ui.tools.SketchingCreationTool"
		},
	    {	id: "com.ibm.sid.ui.sketch.actions.AddColumnToToolBar",
			factory: "com.ibm.rdm.ui.gef.palette.EMFCreationFactory",
	        tool: "com.ibm.sid.ui.tools.SketchingCreationTool"
		},
		{	id: "com.ibm.sid.ui.sketch.actions.AddWindowToToolBar",
			factory: "com.ibm.rdm.ui.gef.palette.EMFCreationFactory",
			tool: "com.ibm.sid.ui.tools.SketchingCreationTool"
		},
		{ 	id: "com.ibm.sid.ui.sketch.actions.AddPanelToToolBar",
			factory: "com.ibm.rdm.ui.gef.palette.EMFCreationFactory",
			tool: "com.ibm.sid.ui.tools.SketchingCreationTool"
		},
		{	id: "com.ibm.sid.ui.sketch.actions.AddGroupBoxToToolBar",
			factory: "com.ibm.rdm.ui.gef.palette.EMFCreationFactory",
			tool: "com.ibm.sid.ui.tools.SketchingCreationTool"
		},
		{	id: "com.ibm.sid.ui.sketch.actions.AddImageToToolBar",
			factory: "com.ibm.rdm.ui.gef.palette.EMFCreationFactory",
			tool: "com.ibm.sid.ui.tools.PromptedCreationTool"
		},
		{	id: "com.ibm.sid.ui.sketch.actions.AddPartToToolBar",
			factory: "com.ibm.rdm.ui.gef.palette.EMFCreationFactory",
			tool: "com.ibm.sid.ui.tools.PromptedCreationTool"
		},
		{	id: "com.ibm.sid.ui.sketch.actions.AddSketchToToolBar",
			factory: "com.ibm.rdm.ui.gef.palette.EMFCreationFactory",
			tool: "com.ibm.sid.ui.tools.PromptedCreationTool"
		}];
	}
});
}());