/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

(function(){
dojo.provide("com.ibm.rdm.web.rrcx.artifact.story.StoryboardHandler");

dojo.require("com.ibm.rdm.web.rrcx.artifact.story.StoryboardEditor");
dojo.require("dojox.xml.parser");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.rrcx.artifact.RRCXArtifactHandler"); 
dojo.require("com.ibm.rdm.web.rrcx.artifact.story.ContextMenuDefinition");
dojo.require("com.ibm.rdm.web.rrcx.artifact.story.StoryboardContentViewer");
dojo.require("com.ibm.rdm.web.common.MimeTypes");

var viewerutils = com.ibm.rdm.web.viewer.viewerutils;
var StoryboardContentView = com.ibm.rdm.web.rrcx.artifact.story.StoryboardContentViewer;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;

/**
   This is the object that the extension providers have to implement
 */
dojo.declare("com.ibm.rdm.web.rrcx.artifact.story.StoryboardHandler", com.ibm.rdm.web.rrcx.artifact.RRCXArtifactHandler, {
	
	_goToFrameListHandle: null,
	_storyboardContentView: null,
	
    _initializeResource: function(resourceObject){
		
		var self = this;
		var blankStory = dojo.xhrGet({
			
			url: dojo.moduleUrl("com.ibm.rdm.web.rrcx", "artifact/story/data/Story") ,
			sync: true,
			handleAs: "text",			
			load: function(response, ioArgs) { 
    			var text = response;
				text = dojo.string.substitute(text, self._generateUuids(2));
				resourceObject.setContent(text);
			}
		});
    },
	
    instantiateResource : function(params)
    {
    	
    		var resourceObject  = this.inherited(arguments);
    		resourceObject._idAttribute = resourceObject._idAttribute.concat( ["key"]);
    		return resourceObject;
    },
    
	viewRRCXResource: function(resourceViewerWidget, webPageHandle) {
		var self = this;
		this.cleanupFromEdit(resourceViewerWidget);
		
		var theResourceObj = resourceViewerWidget.getResourceObject();
		var resourceContent = theResourceObj.getContent();
		var rrcDom = dojox.xml.parser.parse(resourceContent);
		
		this._storyboardContentView = new StoryboardContentView({
			resourceViewerWidget: resourceViewerWidget,
			storyboardHandler: this,
			resourceViewer: resourceViewerWidget.resourceViewer,
			resourceURI: resourceViewerWidget.resourceURI,
			storyContent: rrcDom,
			htmlContentToUseWhenLoading: this._getHtmlContentToUseWhenLoadingViewers,
			onFrameSelected:
				function(frameTitle, frameDescription, inheritsFromResourceURI, inheritsFromDisplayName) {
					self._storyboardFrameSelected(resourceViewerWidget.resourceViewer, 
						frameTitle, frameDescription, inheritsFromResourceURI, inheritsFromDisplayName, false);
				}
		});

		this._goToFrameListHandle = dojo.connect(resourceViewerWidget.resourceViewer._goToFrameListAnchor, "onclick", this, function() {
			this._storyboardContentView.showFrameList();
			var resourceViewer = resourceViewerWidget.resourceViewer;
			resourceViewer._goToFrameListTD.style.display = "none";
			resourceViewer._frameTitle = null;
			resourceViewer._frameDescription = null;
			resourceViewer._updateTitle(resourceViewer._originalTitle);
			resourceViewer.setDescription(resourceViewer._originalDescription);
		});
		
		this._storyboardContentView._handleLoadResponse(rrcDom, resourceViewerWidget.resourceViewer.revision);

		this.resourceViewerWidget._contentPane.attr('content', this._storyboardContentView.domNode);
		this.resourceViewerWidget.setDocumentHasLoaded(true);
		this.resourceViewerWidget.setDimensions();
	},

	//TODO pieces of this care are from RRCXArtifactHandler - can share some of it?
	loadSingleFrame: function(resourceViewerWidget, contentPane, frameIndex) {
	 	var onLoaded = dojo.hitch(this, function (resourceObj){
				var content = "";
				content = viewerutils.transformImageGenerationDocument(resourceObj.ioArgs.xhr.responseText);
				contentPane.domNode.style.position = "absolute";
				contentPane.attr('content', content);

				var idnodes = dojo.query("[id]", contentPane.domNode);
				this.resourceViewerWidget.connectIdNodeOnClick(idnodes);

				this.resourceViewerWidget.setDocumentHasLoaded(true);
				this.resourceViewerWidget.setDimensions();
				this._addRichHovers();
			});
			this.revision = resourceViewerWidget.resourceViewer.revision;
			var urlToUse = resourceViewerWidget._getConversionURL(
					this.resourceViewerWidget.resourceURI, com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_STORY_BOARD, this.revision, frameIndex);
			var retVal = RDMClient.doGet({
				uri: urlToUse,
				skipErrors: true
			});
			retVal.addCallback(this, onLoaded);
		
	 },

	editingArtifact: function() {
		this.cleanup();
	},
	 
	 //removes any elements from the UI from read only mode. 
	 cleanup : function() {
		if (this._storyboardContentView) {
			this._storyboardContentView.destroyRecursive();
			this._storyboardContentView = null;
		}
	 	if (this._goToFrameListHandle) {
			dojo.disconnect(this._goToFrameListHandle);
		}
		var resourceViewer = this.resourceViewerWidget.resourceViewer;
		resourceViewer._goToFrameListTD.style.display = "none";
		resourceViewer._updateTitle(resourceViewer._originalTitle);
		resourceViewer._frameTitle = null;
		resourceViewer.setDescription(resourceViewer._originalDescription);
	 },
	 
	 cleanupFromEdit : function(resourceViewerWidget) {
	 	//make sure we clean up from edit mode on entering view.  Possible we never went to edit mode
		var resourceViewer = resourceViewerWidget.resourceViewer;
		if (!resourceViewer) return;
		resourceViewer._goToFrameListTD.style.display = "none";
		resourceViewer._updateTitle(resourceViewer._originalTitle);
		resourceViewer._frameTitle = null;
		resourceViewer.setDescription(resourceViewer._originalDescription);
	 },
	 
	 setFrameName : function(newFrameName) {
		this.pluginEditorHandle.renameFrame(newFrameName);
	 },

	 
	 updateFrames: function(frameInfo, editor) {	
	 	if (frameInfo.showFrameList) {
			//hide link to frames
			editor.hideFrameNavigation();
			this.cleanupFromEdit(this.resourceViewerWidget);

		} else {
			//show link to frames
			editor.showFrameNavigation();	
		}
		if (frameInfo.frameTitle) {
	 		this._storyboardFrameSelected(this.resourceViewerWidget.resourceViewer, frameInfo.frameTitle, frameInfo.frameDescription, 
	 				null, null, true);
	 	} else {
			//TODO if no title, just cleanup 
		}
	 },
	 
	 doUpdateRemoveMasterAction: function(enabled, editor) {
		 if (enabled) {
	    	// Show Remove Master action
	    	editor.showRemoveMaster();
		 } else {
	    	// Hide Remove Master action
	    	editor.hideRemoveMaster();
		 }
	  },
	 
	 _getHtmlContentToUseWhenLoadingViewers: function() {
		return (
			"<p style=\"color:#999999; margin:20px;\">"
			+ this._messages["waitForContentMessage"]
			+ "</p>"
		);
	},
	
	_storyboardFrameSelected: function(resourceViewer, frameTitle, frameDescription, inheritsFromResourceURI, inheritsFromDisplayName, fromEdit) {
		resourceViewer._storyboardFrameSelected(frameTitle, frameDescription, inheritsFromResourceURI, inheritsFromDisplayName, fromEdit);
	},
    
    getContextMenuContributions: function(){
    	var menu = new com.ibm.rdm.web.rrcx.artifact.story.ContextMenuDefinition();
		return menu.getItems();
    },
	createEditor : function (args){
		return new com.ibm.rdm.web.rrcx.artifact.story.StoryboardEditor(args);
	}
});
}());