/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

(function(){
dojo.provide("com.ibm.rdm.web.rrcx.common.PluginEditor");

dojo.require("com.ibm.rdm.web.ext.ArtifactEditor");
dojo.require("dijit._Widget"); 
dojo.require("com.ibm.rdm.web.rrcx.common.Stylebar");

/**
 * functions common to all current editors
 *   1) _stylebar if it exists
 *   2) invokeAction 
 */ 

dojo.declare("com.ibm.rdm.web.rrcx.common.PluginEditor", 
	[dijit._Widget, com.ibm.rdm.web.ext.ArtifactEditor], 
{ 
	pluginHandle : null,
	_firstRowToolbar: null,
	_paletteToolbar: null,
	_stylebar : null,
	_toDestroy: [],

	postCreate : function(){
		this.inherited(arguments);
		dojo.addClass(this.domNode, "rrcx-PluginEditor");
		this._stylebar&&(this._stylebar.pluginHandle = this.pluginHandle);	
	},
	
	invokeAction : function (event){
		if (!this.pluginHandle){
			console.warn("PluginEditor.invokeAction - no pluginHandle");
			return;
		}
		var widget=dijit.getEnclosingWidget(event.currentTarget);
		if (!widget){
			console.warn("PluginEditor.invokeAction - null widget");
			return;
		}
		
		if (widget.id.indexOf(this.id)==0){
			this.pluginHandle.invokeEditorAction(widget.id.substr(this.id.length));			
		}
		else{
			console.warn("PluginEditor.invokeAction - invalid widget id:"+widget.id);
		}
	},
	
	setAvailableFonts : function (names, sizes){
		this._stylebar&&this._stylebar.setFonts(names, sizes);
	},
	
	updateStylebar : function (items){
		this._stylebar&&this._stylebar.update(items);
	},
	
	uninitialize : function() {
		
		var that = this;
		
		dojo.forEach(this._startupWidgets, function(object){
			that._toDestroy.push(object);
		});
			
		this.inherited(arguments);
	},
	
	destroy : function() {
		
		this.inherited(arguments);
		
		dojo.forEach(this._toDestroy, function(object){
			if (object.destroyRecursive && !object._beingDestroyed){
				object.destroyRecursive();
			}
			else if (object.destroy && !object._beingDestroyed){
				object.destroy();
			}
			delete (object);
		});
	}
});
}());