/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function(){
dojo.provide("com.ibm.rdm.web.rrcx.common.PluginEditorHandle"); 

dojo.require("com.ibm.rdm.web.rrcx.common.communication.CommunicationChannel"); 
dojo.require("com.ibm.rdm.web.common.StatusDisplay");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.require("com.ibm.rdm.web.common.MessageHandler");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.ext.Request");
dojo.require("com.ibm.rdm.web.ext.Response");
dojo.require("com.ibm.rdm.web.ext.Status");
dojo.require("com.ibm.rdm.web.ext.PluginDetector");

dojo.require("com.ibm.rdm.web.rrcx.common.widgets.PluginContentPane");
dojo.require("com.ibm.rdm.web.ext.ArtifactHandler");
dojo.require("com.ibm.rdm.web.module.EditMenu");
dojo.require("com.ibm.rdm.web.module.Operation");
dojo.require("com.ibm.rdm.web.commenting.EventTopics");
dojo.require("com.ibm.rdm.web.notifications.ChangeNotifier");
dojo.require("com.ibm.rdm.web.notifications.EventTopics");
dojo.require("com.ibm.rdm.web.parsers.xml2json");
dojo.require("jazz.ui.internal.MegaMenu");
dojo.require("net.jazz.ajax.ui.internal.HoverMenu");
dojo.require("com.ibm.rdm.web.model.resources.ResourceManager");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");

dojo.require("dojo.dnd.Source"); 
dojo.require("dojo.hash");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.web.rrcx.common", "PluginEditorHandle");

var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;
var StatusDisplay = com.ibm.rdm.web.common.StatusDisplay;
var PluginDetector = com.ibm.rdm.web.ext.PluginDetector;
var MimeTypes = com.ibm.rdm.web.common.MimeTypes;
var MimeTypeRegistry = com.ibm.rdm.web.common.MimeTypeRegistry;
var MegaMenu = jazz.ui.internal.MegaMenu;
var HoverMenu = net.jazz.ajax.ui.internal.HoverMenu;
var ChangeNotifier = com.ibm.rdm.web.notifications.ChangeNotifier;
var EventTopics = com.ibm.rdm.web.commenting.EventTopics;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;

// dom node ids
var plugin_id = 'plugin';
var cbWndExtra = 4; // This comes from RRCXPlugin::createJavaMessageWindow in RRCXPlugin.cpp

/**
   
This is the plugin object that the browser interacts with. for each resource
   there is an instance of this
 */
dojo.declare("com.ibm.rdm.web.rrcx.common.PluginEditorHandle", null, {

    _resourceViewerWidget: null,
    _webPageHandle: null,
	_communicationChannel: null,
    _artifactHandler: null,
    _messages: dojo.i18n.getLocalization("com.ibm.rdm.web.rrcx.common", "PluginEditorHandle"),
    _callbackRegistered: false,
    _closeHandler: null,
    _resizeHandler: null,
    _focusHandler: null,
    _location: null,
    _contentPane: null,
	_editor: null,
	_pluginLoaded: false,
	_installing: false,
	_cacheListener: null,
	
    // Constructor
    constructor: function(webPageHandle, artifactHandler) {
        this._webPageHandle = webPageHandle;
        this._artifactHandler = artifactHandler;
    },
	
    // Public methods
    
	getCommunicationChannel : function() {
		return this._communicationChannel;
	},
	
    // Method used to close the application window
    shutdown: function(e)
    {
    	if (this._resourceViewerWidget.resourceViewer.isDirty()) {
    		dojo.stopEvent(e);
    		e.returnValue = this._messages.shutdownBrowserMessage;
    		return mesg;     	
    	}
     
    	try {
    		this._destroy();
    	} catch (e) {
    		ErrorHandler.debug("Exception: " + e);
			com.ibm.rdm.web.common.MessageHandler.showError("8000", [], e);
    	}
    },    
    
    // Method to close the active editor and close the application window
    closeEditor: function()
    {
		StatusDisplay.hideStatusMessage();
    	if (this._resourceViewerWidget.resourceViewer.isDirty()) {
        	var result = confirm(this._messages.exitMessage);
        	if ( result == true ) {
        		try {
            		this._destroy();
            	} catch (e) {
            		ErrorHandler.debug("Exception: " + e);
 					com.ibm.rdm.web.common.MessageHandler.showError("8000", [], e);
           	}

            	return true;
        	} else {
        		return false;
        	}
        }
    	
    	try {
    		this._destroy();
    	} catch (e) {
    		ErrorHandler.debug("Exception: " + e);
			com.ibm.rdm.web.common.MessageHandler.showError("8000", [], e);
    	}
    	
        return true;  
    },

    _destroy: function() {
//		no such thing
//    	if (this._getPlugin()) 
//    		this._getPlugin().unregisterCallbackObject (this._webPageHandle);
    	//remove all event subscriptions and handlers first - we don't want to get any events during destruction
    	if (this._resizeHandler) {
    		dojo.disconnect(this._resizeHandler);
    		this._resizeHandler = null;
    	}
    	
    	if (this._focusHandler) {
    		dojo.disconnect(this._focusHandler);
    		this._focusHandler = null;
    	}
    	
		if (this._closeHandler) {
    		dojo.disconnect(this._closeHandler);
    		this._closeHandler = null;
		}

		for ( var i = 0; i < this._subscriptions.length; i++) {
			dojo.unsubscribe(this._subscriptions[i]);
			this._subscriptions[i] = null;
		}
		this._subscriptions = [];
		
		//hide UI
		this._contentPane&&this._contentPane.show(false);
			
		if (this._getPlugin()) {
			if (this._pluginLoaded) {
				this._getPlugin().shutdown();
			}
			dojo.destroy(this._getPlugin());
		}
		
		if (this._editor) {
			this._editor.destroyRecursive();
		}
		
		if (this._contentPane) {
			this._contentPane.destroy(false);
			this._contentPane.destroyRendering(false);
			delete this._contentPane;
		}
		
		if (this._cacheListener) {
			ChangeNotifier.removeListener(this._cacheListener);
		}
		
		dojo.style(this._resourceViewerWidget._contentPane.domNode, "overflow", "");
		this._resourceViewerWidget._contentPane.attr(
				'content',  '<div id="empty"></div>');
	},
    // This gives the plugin an object containing methods that it can call, so that it
    // can communicate with the webpage..
    _registerCallbackObject: function(webPageHandle)
    {
		if(this._callbackRegistered)
			return;
		
    	try {
			if (this._getPlugin()) {
				this._communicationChannel = new com.ibm.rdm.web.rrcx.common.communication.CommunicationChannel(webPageHandle, this);
            	this._getPlugin().registerCallbackObject (this._communicationChannel);
				webPageHandle.setCommunicationChannel(this._communicationChannel);
            	this._callbackRegistered = true;
			}
    	} catch (e) {
    		ErrorHandler.debug("Exception: " + e);
			com.ibm.rdm.web.common.MessageHandler.showError("8000", [], e);
    	}
    },
    
	_initSubscriptions: function() {
		this._subscriptions = [];
		this.subscribe(EventTopics.LINK_SELECTED_TOPIC, "selectFragment");
		this.subscribe(EventTopics.RESOURCE_LOCATION_PANE_SELECTED_TOPIC, "selectFragment");
		this.subscribe(EventTopics.LOCATE_COMMENTED_LOCATIONS_IN_RESOURCE, "highlightComments");
		this.subscribe(EventTopics.LOCATE_LINKED_LOCATIONS_IN_RESOURCE, "highlightLinks");
		this._cacheListener = ChangeNotifier.addListener({
			eventTopic: com.ibm.rdm.web.notifications.EventTopics.ELEMENT_CACHE_UPDATED,
			callbackScope: this,
			callbackFunction: this.elementCacheUpdate,
			resourceUriScope: this._resourceViewerWidget.resourceURI
		});
	},
	
	subscribe: function(topic, method) {
    	var topic = EventTopics.getTopic(topic, this._resourceViewerWidget.resourceURI);
    	ErrorHandler.debug("PluginEditorHandle: subscribe to event -> " + topic);
        this._subscriptions.push(dojo.subscribe(topic, this, method));
    },
	
	elementCacheUpdate: function(event) {
		if (event.notifierParams.resourceUriScope == this._resourceViewerWidget.resourceURI && this._getPlugin() && this.getCommunicationChannel()) {
			var request = new com.ibm.rdm.web.ext.Request();
			request.setRequestMethod("updateElementCounts");
			request.setRequestParameter("highlight", event.notifierParams.type);
			this.getCommunicationChannel().sendRequest(request);
		}
	},
	
	selectFragment: function(resourceLocationId, resourceUri) {
		if (this._getPlugin() && resourceLocationId && resourceLocationId != "" && this.getCommunicationChannel()) {
			var request = new com.ibm.rdm.web.ext.Request();
			request.setRequestMethod("selectFragment");
			request.setRequestParameter("resourceLocationId", resourceLocationId);
			request.setRequestParameter("resourceUri", resourceUri);
			this.getCommunicationChannel().sendRequest(request);
		}
	},
	
    // Method to highlight all links in the plugin
    highlightLinks: function(elementMap) {
		if (this._getPlugin()&& this.getCommunicationChannel()) {
			var request = new com.ibm.rdm.web.ext.Request();
			request.setRequestMethod("highlightLinks");
			request.setRequestParameter("elementMap", elementMap);
			this.getCommunicationChannel().sendRequest(request);
		}
    },

    highlightComments : function(elementMap) {
    	if (this._getPlugin()&& this.getCommunicationChannel()) {
			var request = new com.ibm.rdm.web.ext.Request();
			request.setRequestMethod("highlightComments");
			request.setRequestParameter("elementMap", elementMap);
			this.getCommunicationChannel().sendRequest(request);			
		}
    },

    openResource : function(resource)
    {
    	try {
    		if (!this._getPlugin() || !this._pluginLoaded) {
    			this._pluginLoaded = this._loadPlugin();
    		}
			if (this._pluginLoaded) {
				if (this._webPageHandle) {
					// Setup a callback from the callback to notify the pluginEditorHandle when the editor has finished loading.
					this._webPageHandle.setEditorNotifier(dojo.hitch(this, this._hideLoadingMessage));
				}
				this._getPlugin().openResourceURL(resource.getMimeType(), resource.getResourceURI(), resource.getContent(), document.cookie);
				
				//notifies the handler that the resource is entering edit mode.
				this._artifactHandler && this._artifactHandler.editingArtifact && this._artifactHandler.editingArtifact();
			}
			else {
				this._hideLoadingMessage();
				this._destroy();
				this._resourceViewerWidget.resourceViewer.onCancelButton();
				
				if (this._installing) {
					com.ibm.rdm.web.common.MessageHandler.showInfo("8001");
				}
			}
    	} catch (e) {
    		ErrorHandler.debug("Exception: " + e);
			com.ibm.rdm.web.common.MessageHandler.showError("8000", [], e);
    	}    	
    },

	sleep: function(ms) {
		var endTime = new Date();
		endTime.setTime(endTime.getTime() + ms);
		while (new Date().getTime() < endTime.getTime());	
	},
	
	setCurrentBridge: function() {
		var request = new com.ibm.rdm.web.ext.Request();
		request.setRequestMethod("setCurrentBridge");
		var response = this.getCommunicationChannel().sendRequest(request);
	},
	
	updateResourceContent: function(resource){

		if(!this.getCommunicationChannel())
			return false;		

    	var contentReqObj = new com.ibm.rdm.web.ext.Request();
		contentReqObj.setRequestMethod("getEditorXMLContent");
		contentReqObj.setRequestParameter("firstCall", true);
    	var contentRespObj = this.getCommunicationChannel().sendRequest(contentReqObj);
		if (contentRespObj) {
			var xmlContent = contentRespObj.getResponseValue("xmlContent");
			if (xmlContent == null) {
				var retryCount = 0;
				contentReqObj.setRequestParameter("firstCall", false);
				while (retryCount < 10 && xmlContent == null) {
					contentRespObj = this.getCommunicationChannel().sendRequest(contentReqObj);
					if (contentRespObj) {
						xmlContent = contentRespObj.getResponseValue("xmlContent");
					}
					this.sleep(500);
					retryCount++;
				}
			}
			if (xmlContent != null) {
				resource.setContent(xmlContent);
				return true;
			}
		}
		
		return false;
	},
	
    saveEditor: function(resource)
    {
		if(!this.getCommunicationChannel())
			return;
		
		var deferred = new dojo.Deferred();
		try {			
			var updated = this.updateResourceContent(resource);
			
			if (updated) {			
				var deferredSave = resource.save({refreshAfterSave: false});
				deferredSave.addCallback(this, function(result){
					var request = new com.ibm.rdm.web.ext.Request();
					request.setRequestMethod("setDirty");
					request.setRequestParameter("dirtyState", false);
	    			this.getCommunicationChannel().sendRequest(request);
					deferred.callback(result);
				});
				deferredSave.addErrback(this, function(result) {
					deferred.errback(result);
				});
				return deferred;
			}
			deferred.errback(new Error(this._messages.artifactsUnavailableMessage));
		} 
		catch (e) {
			deferred.errback(e);
		}
   		return deferred;
    },    
/*
    saveEditor: function(resource)
    {
    	var contentReqObj = new com.ibm.rdm.web.ext.Request();
		contentReqObj.setRequestMethod("getEditorXMLContent");
    	var contentRespObj = this.getCommunicationChannel().sendRequest(contentReqObj);
    	if (contentRespObj) {
    		var xmlContent = contentRespObj.getResponseValue("xmlContent");
    		resource.setContent(xmlContent);
    		var deferredSave = resource.save({refreshAfterSave: false});
    		deferredSave.addCallback(this, function() {
				var request = new com.ibm.rdm.web.ext.Request();
				request.setRequestMethod("setDirty");
				request.setRequestParameter("dirtyState", false);
    			this.getCommunicationChannel().sendRequest(request);
    		});
    		return deferredSave;
    	}
    	return null;
    },
*/
    saveEditorShortcut: function() {
    	this._resourceViewerWidget.resourceViewer.onSave(false);
    },

    browserSelectionChanged: function(elementId)
    {
    	if(this.getCommunicationChannel()) {
			var request = new com.ibm.rdm.web.ext.Request();
			request.setRequestMethod("browserSelectionChanged");
			request.setRequestParameter("elementId", elementId);
			this.getCommunicationChannel().sendRequest(request);
    	}
    },
    
    setUIEnabled: function(enabled)
    {
    	if(this.getCommunicationChannel()) {
	    	var request = new com.ibm.rdm.web.ext.Request();
			request.setRequestMethod("setUIEnabled");
			request.setRequestParameter("enabled", enabled);
	    	this.getCommunicationChannel().sendRequest(request);
    	}
    },
    
    setResourceViewerWidget: function(resourceViewerWidget) {
    	this._resourceViewerWidget = resourceViewerWidget;
		this._initSubscriptions();
    },
    
	/*
	 * called by the plugin.
	 */
	getPaletteDefinition: function(request){
		var items = this._editor?this._editor.getPluginItems():[];
       	if(this.getCommunicationChannel()) {
	        var respObj = new com.ibm.rdm.web.ext.Response();
	        respObj.setId(request.getId());
	        respObj.setResult(items);
	        this.getCommunicationChannel().sendResponse(respObj, request);
       	}
	},
    createContextMenu: function(request){
        var actionStates = request.getRequestParameter("actionStates");
        var event = request.getRequestParameter("event");
        this._createContextMenu(actionStates, event.x, event.y);
    },
    
    getScreenFlowResourceProps: function(request){
    	var URI = request.getRequestParameter("resourceURI");
    	
    	var resourceInfoDeferred = com.ibm.rdm.web.query.StoredQueryClient.executeStoredQuery({
			queryId: com.ibm.rdm.web.query.StoredQueryClient.QUERYMAP.fetchProjectETagAndFormat,
			queryArgs: {"resource": [URI]},
    		parser: function(response) {
				return response;
    		}
		});
    	resourceInfoDeferred.addCallback(this, function(response){
			
			var feedParser = new com.ibm.rdm.web.parsers.xml2json(response.doc);
			var parsedFeed = feedParser.parse()["feed"];
			for(var entryid in parsedFeed) { 
				if (entryid.indexOf("entry") == 0) {
					var entry = parsedFeed[entryid].content.result;
					var queryResults = {};
					for(var bindingid in entry) {
						if (bindingid.indexOf("binding")==0) {
							var propertyName = entry[bindingid].xmlAttributes.name;
							if (entry[bindingid].uri) {
								queryResults[propertyName] = entry[bindingid].uri.value;
							} else if (entry[bindingid].literal){
								var etag = entry[bindingid].literal.value;
								queryResults[propertyName] = etag.substring(1, etag.length-1); //Strip off quotes
							}
						}
					}
				}
			}
			
			var contentType;
			var mimeType = MimeTypeRegistry.getMimeType(queryResults.ArtifactFormat, queryResults.resourceContext);
			if (mimeType.name === "Screen Flow")
				contentType = MimeTypes.CONTENT_TYPE_SCREEN_FLOW;
			else if (mimeType.name === "User Interface Sketch")
				contentType = MimeTypes.CONTENT_TYPE_SKETCH;
			
			if(this.getCommunicationChannel()) {
				var respObj = new com.ibm.rdm.web.ext.Response();
	            respObj.setId(request.getId());
	            respObj.setResponseValue("contentType", contentType);
	            respObj.setResponseValue("etag", queryResults.etag);
	            this.getCommunicationChannel().sendResponse(respObj, request);
			}
		});
		
		return true;
    },
        
    // Private methods
    
	_hideLoadingMessage: function() {
    	dojo.destroy(this._resourceViewerWidget._contentPane.domNode.firstChild);

		if (this._contentPane != null) {
			this._contentPane.handleEditorLoaded();
			this._contentPane.show();
			this._resizePlugin();
		}    	
				
		StatusDisplay.hideStatusMessage();
	},

    // Retrieves the plugin object 
    _getPlugin: function()
    {	
        return dojo.byId(plugin_id);
    },
    
    _loadMessage: function() {
    	// In an attempt to have the loading message appear earlier we create 
    	// the loading message as soon as we hit edit and add it to the
    	// Resource Viewer Widget's content pane. This message will then be replaced
    	// by the PluginContentPane widget we create in _loadPlugin.
    	StatusDisplay.showStatusMessage(this._messages.loadingEditor);
    	
    	var messageDiv = dojo.create("div",
				{
					style:	{"visibility": "hidden", 
    							"height": this._resourceViewerWidget._heightToUse + "px", "width": "100%", 
    							"overflow": "hidden", "background": "#E5E5E5", "cursor": "wait"}
				});
		var message = dojo.create("p",
				{
					style:	{"color": "#999999", "margin": "20px"},
					innerHTML: this._messages.waitForContentMessage
				});
		messageDiv.appendChild(message);
		
		this._resourceViewerWidget._contentPane.domNode.appendChild(messageDiv);
    	
    	dojo.destroy(this._resourceViewerWidget._contentPane.domNode.firstChild);
    	
    	dojo.style(messageDiv, {"visibility": "visible"});
    },
    
    _loadPlugin: function() {
		/*
		 * Check if the plugin exists and handle installation if not already installed
		 */
    	if (!PluginDetector.detectRRC30(true)) {
    		//Check if the OS and/or Browser version is supported, so the install can start
    		if (PluginDetector.canInstall) {
    			// Didn't find the plugin, start an install process
    			this._handleInstall();
    		}
  			// this._resourceViewerWidget.resourceViewer.handleRefresh();
 	  		return false;
  		}
    	
		// First add our loading message, then load the plugin which will replace that message.
		this._loadMessage();
		
    	this._contentPane = new com.ibm.rdm.web.rrcx.common.widgets.PluginContentPane();

		this._editor=this._artifactHandler.createEditor({pluginHandle:this});
		this._editor&&this._contentPane._toolbar.appendChild(this._editor.domNode);

    	/*
    	 * TODO deal with this
    	 * Temporary fix to disable DnD from the toolbar to prevent browser crash 
    	 *
        var ribbonbarDnD = new dojo.dnd.Source(this._ribbonbar.domNode, {isSource: false});
		*/
		
    	var viewerNode = this._resourceViewerWidget._contentPane.domNode;
    	
    	dojo.style(viewerNode, "overflow", "hidden");
    	this._resourceViewerWidget._contentPane.scroll = "no";
    	dojo.place(this._contentPane.domNode, viewerNode, "last");
    	
    	var resizeCallbackHandler = dojo.hitch(this, this._resizePlugin);
    	
    	this._resizeHandler = dojo.connect(this._resourceViewerWidget._contentPane, "resize", resizeCallbackHandler);
    	
    	var onFocus = dojo.hitch(this, function onFocus(){
    		if(this._getPlugin() && this.getCommunicationChannel()) {
    			this.setCurrentBridge();
    		}
    	});
    	
    	if (/*@cc_on!@*/false) { // check for Internet Explorer
    		this._focusHandler = dojo.connect(document, "onfocusin", onFocus);
    	} else {
    		this._focusHandler = dojo.connect(window, "onfocus", onFocus);
    	}
    	
    	this._closeHandler = dojo.connect(this._resourceViewerWidget.resourceViewer, "onClose", this, "onClose");
        
    	this._registerCallbackObject(this._webPageHandle);
		return true;
    },
	 
	onClose : function(){
		this._destroy();
	},

	_installPopUp: function(url) {
		var installPage = window.open(url, 'installer', 'toolbar=0,scrollbars=0,location=0,statusbar=1,menubar=0,resizable=1,width=400,height=300');
		return installPage;
	},
    
	_handleInstall: function() {
		//summary: Redirect the user to the download location, and trigger the download
			
		this._installing = false;
            
            // Prompt if the user wants to download the plugin
		if (confirm(this._messages.downloadMessage)) {				
			var jsResult;
			var xhrArgs = {
				handleAs: "text"
			};
			
			var deferred = RDMClient.doGet({
				uri: com.ibm.rdm.web.pages.WebUIinitializer.getDataServerRoot() + "/addons/browserAddOnInfo.js",
				skipErrors: true, 
				xhrArgs: xhrArgs
			});
			
			this._installing = true;
			deferred.addCallback(this, function(response) {
		        // Assigns the hash and plugin name from the browserAddOnInfo 
				try {
					eval(response.doc);
			        var extensionHash = (dojo.isIE ? browserAddOnInfo.iexplorer.hash : browserAddOnInfo.firefox.hash);
			        var extensionArtifact = (dojo.isIE ? browserAddOnInfo.iexplorer.artifact : browserAddOnInfo.firefox.artifact);
					
			        // Values used for the FF install
			        var params = {
	    				"RRC Browser Add-On Installation": { 
	    					URL: com.ibm.rdm.web.pages.WebUIinitializer.getDataServerRoot() + '/addons/' + extensionArtifact,
	    					IconURL: com.ibm.rdm.web.pages.WebUIinitializer.getDataServerRoot() + '/addons/install.png',
	    					Hash: extensionHash,
		    				toString: function () { return this.URL; }
		    			}
		    		};
				
					if (net.jazz.ajax.ui.PlatformUI.getWorkbench().shouldVetoNavigation(params.toString()) == false) {
						if (dojo.isFF) {
							// Uses the default plugin install for FF 3.6 and below
							InstallTrigger.install(params);
						}
						else 
							if (dojo.isIE) {
								// Handle creation of an object tag in the DOM to start the installation process
								// Also need to set a parameter so that we know this is in install mode until it completes
								var installPage = this._installPopUp('addons/ieInstall.html');
								installPage.focus();
							}
					}
				}
				catch (ex) {
					com.ibm.rdm.web.common.MessageHandler.showError("8002");
					this._installing = false;
				}			
			});
			deferred.addErrback(this, function() {
				com.ibm.rdm.web.common.MessageHandler.showError("8002");
				this._installing = false;
			});
		}
	},
	
    _resizePlugin: function() {
		//35346 - need to adjust height, otherwise lower portion of editor is cut-off. in IE scrollHeight of the toolbar 
		// is the adjustment value. In FF all heights are 0, need to compute FF specific adjustment value
    	
		if (this._getPlugin()) {
			var heightAdjustment = 57;
			this._getPlugin().setAttribute('height', this._resourceViewerWidget._heightToUse - heightAdjustment);
			this._getPlugin().resize();
		}
		
	},
    // update font style values with the ones supplied with request from java
	contributeFontItems: function(request){
		this._editor&&this._editor.setAvailableFonts(request.requestData.names, request.requestData.sizes);
	},
	
	editorSelectionChanged: function(request){
		var element_uri = request.requestData.selection;
		
	},
	
	fragmentSelected: function(request) {
		if (request && request.requestData && request.requestData.fragmentInfo) {
			var elementId = request.requestData.fragmentInfo.elementId;
			var elementDescription = request.requestData.fragmentInfo.elementDescription;
			if (elementId) {
				ChangeNotifier.notifyListeners({
					eventTopic: com.ibm.rdm.web.notifications.EventTopics.ELEMENT_SELECTED,
					resourceUriScope: this._resourceViewerWidget.resourceURI,
					elementId: elementId,
					elementDescription: elementDescription
				});
			}
		}
	},
	
	renameFrame: function(name){
		if (!this.getCommunicationChannel()) {
			console.warn("PluginEditorHandle.renameFrame:" + name + " - no communication Chanel");
			return;
		}
		var request = new com.ibm.rdm.web.ext.Request();
		request.setRequestMethod("renameFrame");
		request.setRequestParameter("name", name);
		this.getCommunicationChannel().sendRequest(request);
	},
	
	updateStoryboardFrame: function(request) {
		this._artifactHandler && this._artifactHandler.updateFrames(request.requestData.frameInfo, this._editor);
	},
	
	updateRemoveMasterAction: function(request) {
		this._artifactHandler && this._artifactHandler.doUpdateRemoveMasterAction(request.requestData.enabled, this._editor);
	},
	
	// style  button has been activated - send event to java to handle
	handleToolbarEvent : function(eventId, value) {
		if (!this.getCommunicationChannel()) {
			console.warn("PluginEditorHandle.handleToolbarEvent:" + id + ", " + value + " - no communication Chanel");
			return;
		}
		var request = new com.ibm.rdm.web.ext.Request();
		request.setRequestMethod("handleToolbarEvent");
		request.setRequestParameter("eventId", eventId);
		request.setRequestParameter("value", value);
		this.getCommunicationChannel().sendRequest(request);
	},
	
	invokeEditorAction : function(id){
		if (!this.getCommunicationChannel()) {
			console.warn("PluginEditorHandle.invokeEditorAction:" + id + " - no communication Chanel");
			return;
		}
		var request = new com.ibm.rdm.web.ext.Request();
		request.setRequestMethod("invokeEditorAction");
		request.setRequestParameter("id", id);
		this.getCommunicationChannel().sendRequest(request);
	},

	//editor selection changed, update style buttons in the toolbar
	updateStyleButtons: function(request){
		this._editor&&this._editor.updateStylebar(request.requestData.buttons);
	},
	
    _createContextMenu : function(actionStates, x, y) {
    	var items = this._artifactHandler.getContextMenuContributions();
    	// will allow to retrieve details about context menu items - icons, labels, etc.
    	var operations = {};
    	var self = this;
    	dojo.forEach(actionStates, function(action) {
    		// Params: _id, _iconClass, _displayText,
    		// _isEnabledMethod, _executeMethod, _isVisibleMethod
    		var item = self._getItem(action.id, items);
    		if (item == null) return;
    		var image = item.image? dojo.moduleUrl("", item.image).uri: "";
    		var operation = new com.ibm.rdm.web.module.Operation(action.id, image, 
    			function(){
    				return item.label;
    			}, function(){
    				return (action.enabled == "true");
    			}, function(){
    				if(self.getCommunicationChannel()) {
	    				var request = new com.ibm.rdm.web.ext.Request();
						request.setRequestMethod("invokeContextMenuAction");
						request.requestData = action;
	    				self.getCommunicationChannel().sendRequest(request);
    				}
    			}, function(){
    				return (action.enabled == "true");
    			}
    		);
    		operations[action.id] = operation;
    	});
    	
    	var menuLayout = items;
    	this.editMenu = new com.ibm.rdm.web.module.EditMenu({
			node: "plugin",
			menuType: "0",
			rows: [],
			count: 0
		}, operations, menuLayout, "", 1, 1);
    	
		this.editMenu.show({x: x + this._resourceViewerWidget._contentPane.domNode.scrollLeft + cbWndExtra, 
							y: y + this._resourceViewerWidget._contentPane.domNode.scrollTop  + cbWndExtra + 1}, 
						   true);

    },
    
    destroyContextMenu: function() {
    	if (this.editMenu) {
    		this.editMenu.hide();
    		this.editMenu = null;
    	}
    },
    
    _getItem: function(itemId, items) {
    	for (var i = 0; i< items.length; i++) {
    		var operations = items[i].operations;
    		for (var ind = 0; ind < operations.length; ind++) {
    			if (operations[ind].id == itemId) {
    				return operations[ind];
    			}
    		}
    	}
    	return null;
    }
});

}());