/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function(){
dojo.provide("com.ibm.rdm.web.rrcx.common.communication.CommunicationChannel"); 

dojo.require("com.ibm.rdm.web.ext.AbstractCommunicationChannel");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.require("com.ibm.rdm.web.common.MessageHandler");
dojo.require("com.ibm.rdm.web.ext.Request");
dojo.require("com.ibm.rdm.web.ext.Response");
dojo.require("com.ibm.rdm.web.ext.Status");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.web.rrcx.common.communication", "CommunicationChannel");

var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;

/**
   This is the browser object that the plugin interacts with. for each resource
   there is an instance of this
 */
dojo.declare("com.ibm.rdm.web.rrcx.common.communication.CommunicationChannel", com.ibm.rdm.web.ext.AbstractCommunicationChannel, {
	
	_messages: dojo.i18n.getLocalization("com.ibm.rdm.web.rrcx.common.communication", "CommunicationChannel"),
    constructor: function(webPageHandle, editorHandle) {
    },
    
	invokeHTTPMethod: function(request) {
		var methodName = request.getRequestParameter("methodName");
        var methodArgs = request.getRequestParameter("methodArgs");
        if (typeof com.ibm.rdm.web.server.client.RDMClient[methodName] == "function") {
            var deferred = com.ibm.rdm.web.server.client.RDMClient[methodName](methodArgs);
            deferred.addBoth(this, function(methodResponse){
                var status = methodResponse.ioArgs.xhr.status;
                var statusText = methodResponse.ioArgs.xhr.statusText;
                var responseText = methodResponse.ioArgs.xhr.responseText;
                var responseHeaders = methodResponse.ioArgs.xhr.getAllResponseHeaders();
                var response = new com.ibm.rdm.web.ext.Response();
				response.setId(request.getId());
				response.setResponseValue("status", status);
				response.setResponseValue("statusText", statusText);
				response.setResponseValue("responseText", responseText);
				response.setResponseValue("responseHeaders", responseHeaders);
                this.sendResponse(response);
                request.setRepliedTo(true);
            });
        }
	},
	
	_getNativeHandle : function() {
		return this._editorHandle._getPlugin();
	},
	
    sendRequest: function(reqObj) {
		var resultObj = null;
		try {
			var reqString = dojo.toJson(reqObj, true);
			if (this._getNativeHandle()) {
				var result = this._getNativeHandle().sendRequest(reqString);
				resultObj = new com.ibm.rdm.web.ext.Response(result);
			}
    	} catch (e) {
    		ErrorHandler.debug("Exception: " + e);
 			com.ibm.rdm.web.common.MessageHandler.showError("8000", [], e);
	   	} 
    	return resultObj;
	},
	
	sendResponse: function(respObj, reqObj) {
		try {
			var respString = dojo.toJson(respObj, true);
			if (this._getNativeHandle()) {
    			this._getNativeHandle().sendResponse(respString);
			}
			if (reqObj) {
				reqObj.setRepliedTo(true);
			}
    	} catch (e) {
    		ErrorHandler.debug("Exception: " + e);
			com.ibm.rdm.web.common.MessageHandler.showError("8000", [], e);
    	} 
	},
    
	//TODO If necessary, we can modify the request code to determine which target is being addressed
	acceptRequest: function(jsonString) {
        var reqObj = null;
        try {
            reqObj = new com.ibm.rdm.web.ext.Request(jsonString);
            var reqFunct = reqObj.getRequestMethod();
            if (typeof this._webPageHandle[reqFunct] == "function") {
                this._webPageHandle[reqFunct](reqObj);
                return true;
			} else if (typeof this._editorHandle[reqFunct] == "function") {
                this._editorHandle[reqFunct](reqObj);
                return true;
			} else{
				if (typeof this[reqFunct] == "function") {
					this[reqFunct](reqObj);
					return true;
				} else {
					throw new Error(reqFunct + this._messages.methodNotImplemented);
				}
			}
        } 
        catch (err) {
            com.ibm.rdm.web.common.MessageHandler.showError("7381", [], err);
            if (reqObj && reqObj.isSynchronous() && !reqObj.isRepliedTo()) {
                var response = new com.ibm.rdm.web.ext.Response();
                response.setId(reqObj.getId());
                response.setStatus(com.ibm.rdm.web.ext.Status.ERROR);
                response.setMessage(err.toString());
                this.sendResponse(response);
                reqObj.setRepliedTo(true);
            }
        }
        return false;
	},
	
	jvmFailedToStart: function(){
        //mimic cancel button
        this._webPageHandle.getResourceViewer().onCancelButton();
        com.ibm.rdm.web.common.MessageHandler.showError("7382");
    },
	
	osgiFailedToStart: function(){
        //mimic cancel button
        this._webPageHandle.getResourceViewer().onCancelButton();
        com.ibm.rdm.web.common.MessageHandler.showError("7386");
    }
	
});

}());
