/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

dojo.provide("com.ibm.rdm.web.rrcx.common.widgets.ColorPalette");

dojo.require("dijit.ColorPalette");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._Contained");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.web.rrcx.common.widgets", "ColorPalette");

/**
 * A general purpose color palette widget
 * 
 * Much of the complexity is that this can be used in a popup menu so it has to act like a menuitem
 * 
 * TODO fix tabbing (tabbing doesn't work in a popup)
 * 
 * @param {Object} "com.ibm.rdm.web.rrcx.common.widgets.ColorPalette"
 * @param {Object} [dijit._Widget
 * @param {Object} dijit._Templated
 * @param {Object} dijit._Contained]
 * @param {Object} {templatePath:dojo.moduleUrl("com.ibm.rdm.web.rrcx"
 * @param {Object} "common/widgets/templates/ColorPalette.html"
 */

(function() {
	
/**
 * keep track of where a change comes from this
 * is beause we apply changes onBlur :(
 */	
var NONE=0,HEX=1,RGB=2,PALETTE=3;	

var DEFAULT_COLOR = dojo.colorFromArray([0, 0, 0]);

dojo.declare("com.ibm.rdm.web.rrcx.common.widgets.ColorPalette", 
	[dijit._Widget, dijit._Templated, dijit._Contained], {
	templatePath: dojo.moduleUrl("com.ibm.rdm.web.rrcx", "common/widgets/templates/ColorPalette.html"),
	widgetsInTemplate: true,
	_messages : dojo.i18n.getLocalization("com.ibm.rdm.web.rrcx.common.widgets", "ColorPalette"),

	postCreate : function(){
		this._setValues(DEFAULT_COLOR);
		this._lastChange=NONE;
	},
	
	onChange : function(color) {
		this._setValues(dojo.colorFromString(color));
	},

	setColor: function(color){
		this._setValues(dojo.colorFromString(color));
		this._lastChange=NONE;
	},
	
	/**
	 * apply changes
	 * remember that we can get an onBlur before an inputchange
	 */
	onBlur	: function(){
		this._applyChanges();
	},
	_applyChanges : function(){
		var c=null;
		switch (this._changeSource){
			case HEX:
				c = dojo.colorFromString(this._hex.value);
				break;			
			case RGB:
				c = dojo.colorFromArray([this._red.value, this._green.value, this._blue.value]);
				break;			
		}
		c&&this.onChange(c.toHex());
		this._lastChange=NONE; 
	},
	_keypress : function (e){
		switch (e.target) {
			case this._hex:
				this._changeSource=HEX;
				break;
			case this._red:
			case this._green:
			case this._blue:
				this._changeSource=RGB;
				break;
		}
		if (e.keyCode == dojo.keys.ENTER) {
			this._applyChanges();
			this.getParent()&&this.getParent().onExecute&&this.getParent().onExecute(); 
		}
	},
	_inputChange : function (e){
		switch (e.target) {
			case this._hex:
				this._setValues(dojo.colorFromString(this._hex.value));
				break;
			case this._red:
			case this._green:
			case this._blue:
				this._setValues(dojo.colorFromArray([this._red.value, this._green.value, this._blue.value]));
				break;
		}
	},
	_setValues : function (color){
		color = color||DEFAULT_COLOR;
		this._red.value = color.r;
		this._green.value = color.g;
		this._blue.value = color.b;
		this._hex.value = color.toHex();
	},
	_paletteColorChange : function(color){
		this._changeSource=PALETTE;
		this.onChange(color);
		this._setValues(dojo.colorFromString(color)); 
		this.getParent()&&this.getParent().onExecute&&this.getParent().onExecute(); 
	},
	
	/**
	 * the following methods are (or may be) expected
	 * on a dijit menu item.
	 */
	
	onShow: function(){
		this._lastChange=NONE;
	},
	isFocusable: function(){ return true;},
	focus: function(){
		this._palette.focus();
	},
	_setSelected : function (){},
	_onUnhover : function (){},
	_onHover : function (){}

});
})();